<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace RssLib\Objects;

    use InvalidArgumentException;
    use RssLib\Classes\Utilities;
    use RssLib\Interfaces\SerializableObjectInterface;
    use RssLib\Objects\RssChannel\Image;
    use RssLib\Objects\RssChannel\TextInput;

    class RssChannel implements SerializableObjectInterface
    {
        /**
         * @var string
         */
        private $title;

        /**
         * @var string
         */
        private $link;

        /**
         * @var string
         */
        private $description;

        /**
         * @var string|null
         */
        private $language;

        /**
         * @var string|null
         */
        private $copyright;

        /**
         * @var string|null
         */
        private $managing_editor;

        /**
         * @var string|null
         */
        private $web_master;

        /**
         * @var int|null
         */
        private $publish_date;

        /**
         * @var int|null
         */
        private $last_build_date;

        /**
         * @var string[]|null
         */
        private $categories;

        /**
         * @var string|null
         */
        private $generator;

        /**
         * @var string|null
         */
        private $docs;

        /**
         * @var string|null
         */
        private $cloud;

        /**
         * @var int|null
         */
        private $ttl;

        /**
         * @var Image|null
         */
        private $image;

        /**
         * @var TextInput|null
         */
        private $text_input;

        /**
         * @var RssItem[]
         */
        private $items;

        /**
         * @param array $data
         */
        public function __construct(array $data)
        {
            foreach(['title', 'link', 'description'] as $key)
            {
                if(!array_key_exists($key, $data))
                {
                    throw new InvalidArgumentException(sprintf('Missing required key %s for channel', $key));
                }
            }

            $this->title = (string)$data['title'];
            $this->link = (string)$data['link'];
            $this->description = (string)$data['description'];

            if(array_key_exists('language', $data))
            {
                $this->language = (string)$data['language'];
            }

            if(array_key_exists('copyright', $data))
            {
                $this->copyright = (string)$data['copyright'];
            }

            if(array_key_exists('managingEditor', $data))
            {
                $this->managing_editor = (string)$data['managingEditor'];
            }

            if(array_key_exists('webMaster', $data))
            {
                $this->web_master = (string)$data['webMaster'];
            }

            if(array_key_exists('pubDate', $data))
            {
                $this->publish_date = Utilities::parseTimestamp((string)$data['pubDate']);
            }

            if(array_key_exists('lastBuildDate', $data))
            {
                $this->last_build_date = Utilities::parseTimestamp((string)$data['lastBuildDate']);
            }

            if(array_key_exists('category', $data))
            {
                $this->categories = (array)$data['category'];
            }

            if(array_key_exists('generator', $data))
            {
                $this->generator = (string)$data['generator'];
            }

            if(array_key_exists('docs', $data))
            {
                $this->docs = (string)$data['docs'];
            }

            if(array_key_exists('cloud', $data))
            {
                $this->cloud = (string)$data['cloud'];
            }

            if(array_key_exists('ttl', $data))
            {
                $this->ttl = (int)$data['ttl'];
            }

            if(array_key_exists('image', $data))
            {
                $this->image = new Image($data['image']);
            }

            if(array_key_exists('textInput', $data))
            {
                $this->text_input = new TextInput($data['textInput']);
            }

            $this->items = [];
            if(array_key_exists('item', $data))
            {
                foreach($data['item'] as $item)
                {
                    $this->items[] = new RssItem($item);
                }
            }
        }

        /**
         * The name of the channel. It's how people refer to your service. If you have an HTML website that contains
         * the same information as your RSS file, the title of your channel should be the same as the title of your
         * website.
         *
         * @return string
         */
        public function getTitle(): string
        {
            return $this->title;
        }

        /**
         * The URL to the HTML website corresponding to the channel.
         *
         * @return string
         */
        public function getLink(): string
        {
            return $this->link;
        }

        /**
         * Phrase or sentence describing the channel.
         *
         * @return string
         */
        public function getDescription(): string
        {
            return $this->description;
        }

        /**
         * The language the channel is written in
         *
         * @return string|null
         */
        public function getLanguage(): ?string
        {
            return $this->language;
        }

        /**
         * Copyright notice for content in the channel.
         *
         * @return string|null
         */
        public function getCopyright(): ?string
        {
            return $this->copyright;
        }

        /**
         * Email address for person responsible for editorial content.
         *
         * @return string|null
         */
        public function getManagingEditor(): ?string
        {
            return $this->managing_editor;
        }

        /**
         * Email address for person responsible for technical issues relating to channel.
         *
         * @return string|null
         */
        public function getWebMaster(): ?string
        {
            return $this->web_master;
        }

        /**
         * The publication date for the content in the channel
         *
         * @return int|null
         */
        public function getPublishDate(): ?int
        {
            return $this->publish_date;
        }

        /**
         * The last time the content of the channel changed.
         *
         * @return int|null
         */
        public function getLastBuildDate(): ?int
        {
            return $this->last_build_date;
        }

        /**
         * Specify one or more categories that the channel belongs to
         *
         * @return string[]|null
         */
        public function getCategories(): ?array
        {
            return $this->categories;
        }

        /**
         * A string indicating the program used to generate the channel.
         *
         * @return string|null
         */
        public function getGenerator(): ?string
        {
            return $this->generator;
        }

        /**
         * A URL that points to the documentation for the format used in the RSS file
         *
         * @return string|null
         */
        public function getDocs(): ?string
        {
            return $this->docs;
        }

        /**
         * Allows processes to register with a cloud to be notified of updates to the channel, implementing a
         * lightweight publish-subscribe protocol for RSS feeds
         *
         * @return string|null
         */
        public function getCloud(): ?string
        {
            return $this->cloud;
        }

        /**
         * ttl stands for time to live. It's a number of minutes that indicates how long a channel can be cached before
         * refreshing from the source
         *
         * @return int|null
         */
        public function getTtl(): ?int
        {
            return $this->ttl;
        }

        /**
         * Specifies a GIF, JPEG or PNG image that can be displayed with the channel
         *
         * @return Image|null
         */
        public function getImage(): ?Image
        {
            return $this->image;
        }

        /**
         * Specifies a text input box that can be displayed with the channel. More info
         *
         * @return TextInput|null
         */
        public function getTextInput(): ?TextInput
        {
            return $this->text_input;
        }

        /**
         * The items in the channel
         *
         * @return RssItem[]
         */
        public function getItems(): array
        {
            return $this->items;
        }

        /**
         * @inheritDoc
         */
        public function toArray(): array
        {
            return [
                'title' => $this->title,
                'link' => $this->link,
                'description' => $this->description,
                'language' => $this->language,
                'managingEditor' => $this->managing_editor,
                'webMaster' => $this->web_master,
                'pubDate' => $this->publish_date,
                'lastBuildDate' => $this->last_build_date,
                'category' => $this->categories,
                'generator' => $this->generator,
                'docs' => $this->docs,
                'cloud' => $this->cloud,
                'ttl' => $this->ttl,
                'image' => ($this->image) ? $this->image->toArray() : null,
                'textInput' => ($this->text_input) ? $this->text_input->toArray() : null,
                'item' => array_map(static function(RssItem $item) {
                    return $item->toArray();
                }, $this->items)
            ];
        }

        /**
         * @inheritDoc
         */
        public static function fromArray(array $array): SerializableObjectInterface
        {
            return new self($array);
        }
    }