<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace RssLib\Objects\RssChannel;

    use InvalidArgumentException;
    use RssLib\Interfaces\SerializableObjectInterface;

    class Image implements SerializableObjectInterface
    {
        /**
         * @var string
         */
        private $url;

        /**
         * @var string
         */
        private $title;

        /**
         * @var string
         */
        private $link;

        /**
         * @var int|null
         */
        private $width;

        /**
         * @var int|null
         */
        private $height;

        /**
         * @var string|null
         */
        private $description;

        /**
         * Public Constructor
         *
         * @param array $data
         */
        public function __construct(array $data)
        {
            foreach(['url', 'title', 'link'] as $key)
            {
                if(!array_key_exists($key, $data))
                {
                    throw new InvalidArgumentException(sprintf('Missing required key %s for image', $key));
                }
            }

            $this->url = (string)$data['url'];
            $this->title = (string)$data['title'];
            $this->link = (string)$data['link'];

            if(array_key_exists('width', $data))
            {
                $this->width = (int)$data['width'];
            }

            if(array_key_exists('height', $data))
            {
                $this->height = (int)$data['height'];
            }

            if(array_key_exists('description', $data))
            {
                $this->description = (string)$data['description'];
            }
        }

        /**
         * the URL of a GIF, JPEG or PNG image that represents the channel.
         *
         * @return string
         */
        public function getUrl(): string
        {
            return $this->url;
        }

        /**
         * describes the image, it's used in the ALT attribute of the HTML <img> tag when the channel is rendered in HTML.
         *
         * @return string
         */
        public function getTitle(): string
        {
            return $this->title;
        }

        /**
         * the URL of the site, when the channel is rendered, the image is a link to the site.
         *
         * @return string
         */
        public function getLink(): string
        {
            return $this->link;
        }

        /**
         * Optional. The width of the image in pixels
         *
         * @return int|null
         */
        public function getWidth(): ?int
        {
            return $this->width;
        }

        /**
         * Optional. The height of the image in pixels
         *
         * @return int|null
         */
        public function getHeight(): ?int
        {
            return $this->height;
        }

        /**
         * Optional. Specifies the text in the HTML TITLE attribute of the link around the image
         *
         * @return string|null
         */
        public function getDescription(): ?string
        {
            return $this->description;
        }

        /**
         * @inheritDoc
         */
        public function toArray(): array
        {
            return [
                'url' => $this->url,
                'title' => $this->title,
                'link' => $this->link,
                'width' => $this->width,
                'height' => $this->height,
                'description' => $this->description
            ];
        }

        /**
         * @inheritDoc
         */
        public static function fromArray(array $array): Image
        {
            return new self($array);
        }
    }