<?php /** @noinspection PhpMissingFieldTypeInspection */

namespace RssLib\Objects;

    use InvalidArgumentException;
    use RssLib\Classes\Utilities;
    use RssLib\Interfaces\SerializableObjectInterface;
    use RssLib\Objects\RssFeed\Enclosure;
    use RssLib\Objects\RssFeed\Guid;
    use RssLib\Objects\RssFeed\Source;

    class RssItem implements SerializableObjectInterface
    {
        /**
         * @var string|null
         */
        private $title;

        /**
         * @var string|null
         */
        private $link;

        /**
         * @var string|null
         */
        private $description;

        /**
         * @var string|null
         */
        private $author;

        /**
         * @var string[]|null
         */
        private $categories;

        /**
         * @var string|null
         */
        private $comments;

        /**
         * @var Enclosure|null
         */
        private $enclosure;

        /**
         * @var Guid|null
         */
        private $guid;

        /**
         * @var int|null
         */
        private $publish_date;

        /**
         * @var Source|null
         */
        private $source;

        /**
         * RssFeed constructor.
         *
         * @param array $data
         * @return void
         */
        public function __construct(array $data)
        {
            foreach(['title', 'link', 'description'] as $key)
            {
                if(!array_key_exists($key, $data))
                {
                    throw new InvalidArgumentException(sprintf('Missing required key %s for RssFeed', $key));
                }
            }

            $this->title = (string)$data['title'];
            $this->link = (string)$data['link'];
            $this->description = (string)$data['description'];

            if(array_key_exists('author', $data))
            {
                $this->author = (string)$data['author'];
            }

            if(array_key_exists('categories', $data))
            {
                if(is_string($data['categories']))
                {
                    $this->categories = [$data['categories']];
                }
                else
                {
                    $this->categories = (array)$data['categories'];
                }
            }

            if(array_key_exists('comments', $data))
            {
                $this->comments = (string)$data['comments'];
            }

            if(array_key_exists('enclosure', $data))
            {
                $this->enclosure = new Enclosure($data['enclosure']);
            }

            if(array_key_exists('guid', $data))
            {
                $this->guid = new Guid($data['guid']);
            }

            if(array_key_exists('pubDate', $data))
            {
                $this->publish_date = Utilities::parseTimestamp($data['pubDate']);
            }

            if(array_key_exists('source', $data))
            {
                $this->source = new Source($data['source']);
            }
        }

        /**
         * Returns the title of the item.
         *
         * @return string|null
         */
        public function getTitle(): ?string
        {
            return $this->title;
        }

        /**
         * Returns the link of the item.
         *
         * @return string|null
         */
        public function getLink(): ?string
        {
            return $this->link;
        }

        /**
         * Returns the description of the item.
         *
         * @return string|null
         */
        public function getDescription(): ?string
        {
            return $this->description;
        }

        /**
         * Returns the author of the item.
         *
         * @return string|null
         */
        public function getAuthor(): ?string
        {
            return $this->author;
        }

        /**
         * Returns the categories of the item.
         *
         * @return string[]|null
         */
        public function getCategories(): array|null
        {
            return $this->categories;
        }

        /**
         * Returns the comments of the item.
         *
         * @return string|null
         */
        public function getComments(): ?string
        {
            return $this->comments;
        }

        /**
         * Returns the enclosure of the item.
         *
         * @return Enclosure|null
         */
        public function getEnclosure(): ?Enclosure
        {
            return $this->enclosure;
        }

        /**
         * Returns the GUID of the item.
         *
         * @return Guid|null
         */
        public function getGuid(): ?Guid
        {
            return $this->guid;
        }

        /**
         * Returns the publication date of the item.
         *
         * @return int|null
         */
        public function getPublishDate(): ?int
        {
            return $this->publish_date;
        }

        /**
         * Returns the source of the item.
         *
         * @return Source|null
         */
        public function getSource(): ?Source
        {
            return $this->source;
        }

        /**
         * @inheritDoc
         */
        public function toArray(): array
        {
            return [
                'title' => $this->title,
                'link' => $this->link,
                'description' => $this->description,
                'author' => $this->author,
                'categories' => $this->categories,
                'comments' => $this->comments,
                'enclosure' => $this->enclosure?->toArray(),
                'guid' => $this->guid?->toArray(),
                'pubDate' => $this->publish_date,
                'source' => $this->source?->toArray(),
            ];
        }

        /**
         * @inheritDoc
         */
        public static function fromArray(array $array): SerializableObjectInterface
        {
            return new self($array);
        }
    }